import os

class DjangoDriver:

    def wsgi_path(self, directory):
        return self.find('wsgi.py', directory)
    
    def detect(self, directory):
        return os.path.exists(
            os.path.join(directory, 'manage.py')
        )

    def find(self, name, path):
        for root, dirs, files in os.walk(path):
            if name in files:
                return os.path.join(root, name)


import subprocess
import os
import glob
from pathlib import Path


class Command:

    def startup(self):
        for directory in self.get_registered_directories():
            site = directory.split('/')[-2]
            activation_environment = self.find_virtual_environment_activation_file(directory)

            command = f"cd {directory}"
            command += f" && source {activation_environment}"
            command += f" && pip install uwsgi && set -m; nohup uwsgi --socket /tmp/{site}.test.sock --wsgi-file wsgi.py &> /dev/null &"
            subprocess.run(command, shell=True, close_fds=True,
                           env={'PYTHONPATH': f'{directory}'})

    def get_home_path(self):
        return str(Path.home())

    def get_registered_directories(self):
        return glob.glob(os.path.join(self.get_home_path(), 'sites/*/'))

    def find_virtual_environment_activation_file(self, directory):
        return f"{directory}venv/bin/activate"



import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..services.Configuration import Configuration

class KillCommand(CLICommand):
    """
    Kills serving applications

    kill
        {app? : The application name you want to kill}
    """

    def handle(self):
        if self.argument('app'):
            self.kill(self.argument('app'))
        else:
            for directory in self.get_registered_directories():
                app = directory.split('/')[-2]
                self.kill(app)
    
    def kill(self, app):
        self.info(f"Killing app: {app}")

        configuration = Configuration()
        tld = configuration.get('tld')
        socket_directory = configuration.get('socket_directory')
        subprocess.run(f"rm {os.path.join(socket_directory, app)}.{tld}.sock", shell=True)
        self.info("App stopped")  

    def get_registered_directories(self):
        directories = []
        for directory in Configuration().get('directories', []):
            directories += glob.glob(os.path.join(directory, '*/'))
        return directories


import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..services.Configuration import Configuration
from ..settings import PATHS

class InstallCommand(CLICommand):
    """
    Installs Foreman

    install
    """

    def handle(self):
        # Brew update
        self.info('Updating Brew ..')
        subprocess.run("brew update", shell=True)
        # Brew install NGINX
        self.info('Installing NGINX ..')
        subprocess.run("brew install nginx", shell=True)

        with open(os.path.join(self.get_home_path(), '.foreman', 'config.yml'), 'w+') as f:
            pass 
        
        self.info('Stopping nginx ..')
        subprocess.run("sudo nginx -s stop", shell=True)
        with open(os.path.join(PATHS['stubs'], 'nginx.conf')) as f:
            output = f.read()
        
        self.info('Creating foreman config directory .. ')
        subprocess.run("mkdir -p ~/.foreman", shell=True)

        self.info('Moving NGINX config ..')
        output = output.replace('FOREMAN_NGINX_CONF', os.path.join(
            self.get_home_path(), '.foreman/nginx.conf'))
        output = output.replace('SYSTEM_USER', self.current_user())
        
        with open(self.nginx_config_path(), 'w+') as f:
            f.write(output)


        self.info("Moving Foreman config ..")

        with open(os.path.join(PATHS['stubs'], 'foreman.conf')) as f:
            output = f.read()

        output = output.replace('TLD', 'test')

        with open(os.path.join(self.get_home_path(), '.foreman/nginx.conf'), 'w+') as f:
            f.write(output)

        self.info('Restarting nginx ..')
        subprocess.run("sudo nginx", shell=True)
        configuration = Configuration()
        configuration.set('tld', 'test')
        configuration.set('socket_directory', '/tmp')

    def nginx_config_path(self):
        return subprocess.check_output(
            "nginx -V 2>&1 | grep -o '\-\-conf-path=\(.*conf\)' | cut -d '=' -f2", shell=True).decode('utf-8').strip()
    
    def current_user(self):
        return subprocess.check_output("whoami", shell=True).decode('utf-8').strip()
    
    def get_home_path(self):
        return str(Path.home())



import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..services.Configuration import Configuration

class UnregisterCommand(CLICommand):
    """
    Unregisters the current directory from serving Python applications

    unregister
        {directory? : The directory you want to unregister}
    """

    def handle(self):
        if self.argument('directory'):
            directory = self.argument('directory')
        else:
            directory = os.getcwd()

        configuration = Configuration()

        self.info(f'Removing directory: {directory}')
        configuration.remove('directories', [directory])
        self.info('Directory has been deregistered from serving python applications')

        self.call("start")